/****************************************************************************************
***** Macro qui assigne l'indice de dfavorisation de 2011 du Canada ainsi que ses 	*****
***** dclinaisons gographiques (Rgions canadiennes, zones gographiques, grandes *****
***** rgions mtropolitaines de recensement).				 						*****
***** L'assignation est ralise  partir du code postal (obligatoire) et du code	*****
***** municipal (facultatif). La macro est base sur une version simplifie du		*****
***** Fichier de conversion des codes postaux (FCCP+) de Statistique Canada et fait *****
***** appel aux trois fichiers de conversion suivants:								*****
*****  - Fichier de codes postaux uniques (PCCFUNIQ)                                *****
*****  - Fichier de codes postaux non uniques (PCCFDUPS)                           	*****
*****  - Fichier de codes postaux non uniques avec pondration connue (WCF)         *****
*****  cela s'ajoute un quatrime fichier (MUNIC) qui effectue l'historique des	*****
***** codes municipaux au Canada de 1991  2011.									*****
*****																				***** 
***** Les quatre paramtres de la macro sont:                                       *****
*****  - IN: nom du fichier SAS en entre auquel on dsire ajouter l'indice de   	*****
*****        dfavorisation de 2011.						    					*****
*****  - PCODE: nom de la variable du code postal dans le fichier d'entre.    		*****
*****  - MUNIC: nom de la variable du code municipal dans le fichier d'entre. Par  *****
*****           dfaut, la valeur 0 (zro) signifie que l'assignation est ralise  *****
*****           seulement  partir du code postal.                        			*****
*****  - OUT: nom du fichier SAS de sortie, qui sera identique au fichier d'entre 	*****
*****         mais bonifi des deux nouvelles variables.      						*****
*****																				*****
***** Auteur: Denis Hamel, INSPQ                                    				*****
*****         Philippe Gamache, INSPQ                               				*****
***** Dernire modification: 18 octobre 2015                        				*****
*****************************************************************************************/


/****************************************************************************************/
/* 1. tablir le chemin daccs du rpertoire o se trouve les 4 fichiers de donnes    */
%let repertoire = D:\Projets;


/****************************************************************************************/
/* 2. Dterminer les quatre paramtres (par dfaut, munic = 0)						    */
%let in = fichier_depart;
%let pcode = codepostal;
%let munic = 0;
%let out = fichier_assigne;


/****************************************************************************************/
/* 3. Excution de la macro. Ne rien changer  partir d'ici						    	*/
%macro assignation2011(in=, pcode=, munic=, out=);

filename fcp "&repertoire\fcp.txt";
filename pccfuniq "&repertoire\fccpindiceuniq.txt";
filename pccfdups "&repertoire\fccpindicedouble.txt";
filename munic "&repertoire\munic2011.txt";

libname i2011 "&repertoire";

data i2011.wcf;
  infile fcp;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 quintmat 30 quintsoc 31 QuintMatRC 32 QuintSocRC 33 QuintMatZone 34 QuintSocZone 35 
        QuintMatRMR 36 QuintSocRMR 37 zone 38 PR $ 39-40 discord 41 css 42 TGN_AD 5.1;
run;

data i2011.pccfuniq;
  infile pccfuniq;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 quintmat 30 quintsoc 31 QuintMatRC 32 QuintSocRC 33 QuintMatZone 34 QuintSocZone 35 
        QuintMatRMR 36 QuintSocRMR 37 zone 38 PR $ 39-40 discord 41 css 42 TGN_AD 5.1;
run;

data i2011.pccfdups;
  infile pccfdups;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 quintmat 30 quintsoc 31 QuintMatRC 32 QuintSocRC 33 QuintMatZone 34 QuintSocZone 35 
        QuintMatRMR 36 QuintSocRMR 37 zone 38 PR $ 39-40 discord 41 css 42 TGN_AD 5.1;
run;

data i2011.munic;
  infile munic;
  input munic 1-7 mun2011 8-14;
run;

data wcf;
   set i2011.wcf;
   nuniq = _n_;
run;

data pccfuniq;
   set i2011.pccfuniq;
   nuniq = _n_;
run;

data pccfdups;
   set i2011.pccfdups;
   nuniq = _n_;
run;


data fichin;
  %if &munic ne 0 %then %do; 
    length munic 8.; 
  %end;
  set &in;
  numero=_n_;
  nba=ranuni(766);
  pcode=compress(&pcode);
  %if &munic ne 0 %then 
     %do; 
       if substr(pcode,1,1) in ("A") then munic=1000000+mod(&munic,1000000);
       else if substr(pcode,1,1) in ("B") then munic=1200000+mod(&munic,1200000);
       else if substr(pcode,1,1) in ("C") then munic=1100000+mod(&munic,1100000);
       else if substr(pcode,1,1) in ("E") then munic=1300000+mod(&munic,1300000);
       else if substr(pcode,1,1) in ("G","H","J") then munic=2400000+mod(&munic,2400000);
       else if substr(pcode,1,1) in ("K","L","M","N","P") then munic=3500000+mod(&munic,3500000);
       else if substr(pcode,1,1) in ("R") then munic=4600000+mod(&munic,4600000);
       else if substr(pcode,1,1) in ("S") then munic=4700000+mod(&munic,4700000);
       else if substr(pcode,1,1) in ("T") then munic=4800000+mod(&munic,4800000);
       else if substr(pcode,1,1) in ("V") then munic=5900000+mod(&munic,5900000);
       else if substr(pcode,1,1) in ("X") then munic=6100000+mod(&munic,6100000);
       else if substr(pcode,1,1) in ("Y") then munic=6000000+mod(&munic,6000000);
     %end;
run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
data fichin;
  set fichin;
  where zone in (.,0);
run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;


/****************************************************************************************/
/* Assignation de l'indice  partir du code postal et de la municipalit (si munic ne 0)*/

%if &munic ne 0 %then %do;

proc sort data=i2011.munic;
  by munic;
run;

proc sort data=fichin;
  by munic;
run;

data fichin;
  merge i2011.munic(in=in2) fichin(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;

/***** Avec PCCFUNIQ *****/
/*** Canada ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.quintmatrc, pccfuniq.quintsocrc, 
         pccfuniq.quintmatzone, pccfuniq.quintsoczone, pccfuniq.quintmatrmr, pccfuniq.quintsocrmr, pccfuniq.propmunic, pccfuniq.nuniq, 
         pccfuniq.zone, pccfuniq.TGN_AD, pccfuniq.discord, pccfuniq.pr, pccfuniq.css
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode and fichin.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test3;
  by numero;
run;

proc sort data=fichin;
  by numero;
run; 

data appmunic1 nonapp1;
  merge fichin(in=in1) test3(in=in2c);
  by numero;
  if in1*in2c then output appmunic1;
  else if in1 then output nonapp1;
run;

/***** Avec WCF *****/
/*** Canada ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.first, wcf.quintmat, wcf.quintsoc, wcf.quintmatrc, wcf.quintsocrc, wcf.pr,
         wcf.quintmatzone, wcf.quintsoczone, wcf.quintmatrmr, wcf.quintsocrmr, wcf.propmunic, wcf.nuniq, wcf.zone, wcf.TGN_AD, wcf.discord, wcf.css
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test3;
  by numero;
run;

proc sort data=nonapp1;
  by numero;
run; 

data appmunic2 nonapp2;
  merge nonapp1(in=in1) test3(in=in2c);
  by numero;
  if in1*in2c then output appmunic2;
  else if in1 then output nonapp2;
run;

/***** Avec PCCFDUPS *****/
/*** Canada ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfdups.first, pccfdups.quintmat, pccfdups.quintsoc, pccfdups.quintmatrc, pccfdups.quintsocrc, 
         pccfdups.quintmatzone, pccfdups.quintsoczone, pccfdups.quintmatrmr, pccfdups.quintsocrmr, pccfdups.propmunic, pccfdups.nuniq, pccfdups.css,
         pccfdups.zone, pccfdups.TGN_AD, pccfdups.discord, pccfdups.pr
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode and fichin.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test3;
  by numero;
run;

proc sort data=nonapp2;
  by numero;
run; 

data appmunic3 fichin;
  merge nonapp2(in=in1) test3(in=in2c);
  by numero;
  if in1*in2c then output appmunic3;
  else if in1 then output fichin;
run;


%end;

/****************************************************************************************/
/* Assignation de l'indice  partir du code postal seulement							*/

/***** Avec PCCFUNIQ *****/
/*** Canada ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.quintmatrc, pccfuniq.quintsocrc, 
         pccfuniq.quintmatzone, pccfuniq.quintsoczone, pccfuniq.quintmatrmr, pccfuniq.quintsocrmr,pccfuniq.propcp, pccfuniq.nuniq, 
         pccfuniq.zone, pccfuniq.TGN_AD, pccfuniq.discord, pccfuniq.pr, pccfuniq.css
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test3;
  by numero;
run;

proc sort data=fichin;
  by numero;
run; 

data appcp1 nonapp3;
  merge fichin(in=in1) test3(in=in2c);
  by numero;
  if in1*in2c then output appcp1;
  else if in1 then output nonapp3;
run;

/***** Avec WCF *****/
/*** Canada ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf.first, wcf.quintmat, wcf.quintsoc, wcf.quintmatrc, wcf.quintsocrc, wcf.pr,
         wcf.quintmatzone, wcf.quintsoczone, wcf.quintmatrmr, wcf.quintsocrmr, wcf.propcp, wcf.nuniq, wcf.zone, wcf.TGN_AD, wcf.discord, wcf.css
  from fichin, wcf
  where fichin.pcode=wcf.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test3;
  by numero;
run;

proc sort data=nonapp3;
  by numero;
run; 

data appcp2 nonapp4;
  merge nonapp3(in=in1) test3(in=in2c);
  by numero;
  if in1*in2c then output appcp2;
  else if in1 then output nonapp4;
run;

/***** Avec PCCFDUPS *****/
/*** Canada ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfdups.first, pccfdups.quintmat, pccfdups.quintsoc, pccfdups.quintmatrc, pccfdups.quintsocrc, 
         pccfdups.quintmatzone, pccfdups.quintsoczone, pccfdups.quintmatrmr, pccfdups.quintsocrmr, pccfdups.propcp, pccfdups.nuniq, 
         pccfdups.zone, pccfdups.TGN_AD, pccfdups.discord, pccfdups.pr, pccfdups.css
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test3;
  by numero;
run;

proc sort data=nonapp4;
  by numero;
run; 

data appcp3 nonapp5;
  merge nonapp4(in=in1) test3(in=in2c);
  by numero;
  if in1*in2c then output appcp3;
  else if in1 then output nonapp5;
run;


data tout;
  set %if &munic ne 0 %then %do; appmunic1(in=in1) appmunic2(in=in2) appmunic3(in=in3) %end; 
                                 appcp1(in=in4) appcp2(in=in5) appcp3(in=in6) nonapp5(in=in7);
  if in7 then direct=0;
  if in5 then do; 
    direct=1;
  end;

  %if &munic ne 0 %then %do; 
     if in1 then base=1;
     else if in2 then do; 
       direct=1;base=2;
     end;
	 else if in3 then base=3;
   %end;
   if in4 then base=1;
   else if in5 then base=2;
   else if in6 then base=3;

   if in7=1 then do;quintmat = 0;quintsoc = 0;quintmatrc=0;quintsocrc=0;quintmatzone=0;quintsoczone=0;quintmatrmr=0;quintsocrmr=0;zone = 0;css=0;
   end;
   keep numero base quintmat quintsoc quintmatrc quintsocrc quintmatzone quintsoczone quintmatrmr quintsocrmr zone TGN_AD discord CSS pr;
run;

proc sort; 
  by numero; 
run; 

data fichin;
  set &in;
  numero=_n_;
run;

data tout;
  retain TGN_AD discord pr zone CSS QuintMat QuintSoc QuintMatRC QuintSocRC QuintMatZone QuintSocZone QuintMatRMR QuintSocRMR;
  set tout;
run;

data &out;
  merge fichin tout;
  by numero;
  DATE = '20151018';
  *format groupe fgroupe. base fbase.;
  *drop numero;
run;

proc datasets library=work;
  delete corr2 wcf2 pccfuniq2 pccfdups2 test test1 test2 test3 fichin tout appmunic1 appmunic2 appmunic3 appmunic4 appcp1 appcp2 appcp3 appcp4 nonapp1 nonapp2 
         nonapp3 nonapp4 nonapp5 nonapp6 nonapp7 pccfdup2; 
quit;
%mend assignation2011;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));
		 /*** la macrovariable presence = 0 si la variable est absente du fichier ****/
	   %end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;



/***** Exemple *****/
%let in = deathmaster;
%let pcode = CPRESDCD;
%let munic = CMUNRESD;
%let out = fichier_assigne;

%assignation2011(in=&in,pcode=&pcode,munic=&munic,out=&out);
